@@ -46,6 +46,15 @@ context 'Invoker' do
     assert invoker.read_output.empty?
   end
 
+  test 'should allow docdate and doctime to be overridden' do
+    sample_filepath = File.expand_path(File.join(File.dirname(__FILE__), 'fixtures', 'sample.asciidoc'))
+    invoker = invoke_cli_to_buffer %w(-o /dev/null -a docdate=2015-01-01 -a doctime=10:00:00-07:00), sample_filepath
+    doc = invoker.document
+    assert doc.attr?('docdate', '2015-01-01')
+    assert doc.attr?('doctime', '10:00:00-07:00')
+    assert doc.attr?('docdatetime', '2015-01-01 10:00:00-07:00')
+  end
+
   test 'should accept document from stdin and write to stdout' do
     invoker = invoke_cli_to_buffer(%w(-s), '-') { 'content' }
     doc = invoker.document
@@ -185,7 +194,7 @@ context 'Invoker' do
     destination_path = File.expand_path(File.join(File.dirname(__FILE__), 'test_output'))
     sample_outpath = File.join(destination_path, 'sample.html')
     begin
-      FileUtils.mkdir_p(destination_path) 
+      FileUtils.mkdir_p(destination_path)
       # QUESTION should -D be relative to working directory or source directory?
       invoker = invoke_cli %w(-D test/test_output)
       #invoker = invoke_cli %w(-D ../../test/test_output)
@@ -277,6 +286,22 @@ context 'Invoker' do
     end
   end
 
+  test 'should not copy custom stylesheet to target directory if stylesdir is a URI' do
+    destdir = File.expand_path(File.join(File.dirname(__FILE__), 'fixtures', 'output'))
+    sample_outpath = File.join destdir, 'sample-output.html'
+    stylesdir = File.join destdir, 'http:'
+    begin
+      invoker = invoke_cli %W(-o #{sample_outpath} -a linkcss -a stylesdir=http://example.org/styles -a stylesheet=custom.css)
+      invoker.document
+      assert File.exist?(sample_outpath)
+      assert !File.exist?(stylesdir)
+    ensure
+      FileUtils.rm_f(sample_outpath)
+      FileUtils.rmdir(stylesdir) if File.directory? stylesdir
+      FileUtils.rmdir(destdir)
+    end
+  end
+
   test 'should render all passed files' do
     basic_outpath = File.expand_path(File.join(File.dirname(__FILE__), 'fixtures', 'basic.html'))
     sample_outpath = File.expand_path(File.join(File.dirname(__FILE__), 'fixtures', 'sample.html'))
@@ -456,7 +481,7 @@ context 'Invoker' do
   test 'default mode for cli should be unsafe' do
     invoker = invoke_cli_to_buffer %w(-o /dev/null)
     doc = invoker.document
-    assert_equal Asciidoctor::SafeMode::UNSAFE, doc.safe 
+    assert_equal Asciidoctor::SafeMode::UNSAFE, doc.safe
   end
 
   test 'should set safe mode if specified' do
@@ -504,7 +529,7 @@ context 'Invoker' do
       assert !stdout_lines.empty?
       stdout_lines.each {|l| l.force_encoding Encoding::UTF_8 } if Asciidoctor::FORCE_ENCODING
       stdout_str = stdout_lines.join
-      assert stdout_str.include?('Codierungen sind verrückt auf älteren Versionen von Ruby') 
+      assert stdout_str.include?('Codierungen sind verrückt auf älteren Versionen von Ruby')
     ensure
       ENV['LANG'] = old_lang
     end
