@@ -1,4 +1,8 @@
-require 'test_helper'
+# encoding: UTF-8
+unless defined? ASCIIDOCTOR_PROJECT_DIR
+  $: << File.dirname(__FILE__); $:.uniq!
+  require 'test_helper'
+end
 
 context 'Tables' do
 
@@ -12,10 +16,14 @@ context 'Tables' do
 |=======
       EOS
       cells = [%w(A B C), %w(a b c), %w(1 2 3)]
-      output = render_embedded_string input
+      doc = document_from_string input, :header_footer => false
+      table = doc.blocks[0]
+      assert 100, table.columns.map {|col| col.attributes['colpcwidth'] }.reduce(:+)
+      output = doc.convert
       assert_css 'table', output, 1
-      assert_css 'table.tableblock.frame-all.grid-all[style*="width:100%"]', output, 1
-      assert_css 'table > colgroup > col[style*="width:33%"]', output, 3
+      assert_css 'table.tableblock.frame-all.grid-all.spread', output, 1
+      assert_css 'table > colgroup > col[style*="width: 33.3333%"]', output, 2
+      assert_css 'table > colgroup > col:last-of-type[style*="width: 33.3334%"]', output, 1
       assert_css 'table tr', output, 3
       assert_css 'table > tbody > tr', output, 3
       assert_css 'table td', output, 9
@@ -98,6 +106,46 @@ context 'Tables' do
       assert_xpath '/table/tbody/tr/td[2]/p[text()="a | there"]', output, 1
     end
 
+    test 'preserves escaped delimiters at the end of the line' do
+      input = <<-EOS
+[%header,cols="1,1"]
+|====
+|A |B\\|
+|A1 |B1\\|
+|A2 |B2\\|
+|====
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > thead > tr', output, 1
+      assert_css 'table > thead > tr:nth-child(1) > th', output, 2
+      assert_xpath '/table/thead/tr[1]/th[2][text()="B|"]', output, 1
+      assert_css 'table > tbody > tr', output, 2
+      assert_css 'table > tbody > tr:nth-child(1) > td', output, 2
+      assert_xpath '/table/tbody/tr[1]/td[2]/p[text()="B1|"]', output, 1
+      assert_css 'table > tbody > tr:nth-child(2) > td', output, 2
+      assert_xpath '/table/tbody/tr[2]/td[2]/p[text()="B2|"]', output, 1
+    end
+
+    test 'should treat trailing pipe as an empty cell' do
+      input = <<-EOS
+|====
+|A1 |
+|B1 |B2
+|C1 |C2
+|====
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > tbody > tr', output, 3
+      assert_xpath '/table/tbody/tr[1]/td', output, 2
+      assert_xpath '/table/tbody/tr[1]/td[1]/p[text()="A1"]', output, 1
+      assert_xpath '/table/tbody/tr[1]/td[2]/p', output, 0
+      assert_xpath '/table/tbody/tr[2]/td[1]/p[text()="B1"]', output, 1
+    end
+
     test 'should auto recover with warning if missing leading separator on first cell' do
       input = <<-EOS
 |===
@@ -124,7 +172,7 @@ A | here| a | there
       EOS
       output = render_embedded_string input
       assert_xpath '//tbody/tr/td[1]/p[text()="Cool new show"]', output, 1
-      assert_xpath %(//tbody/tr/td[2]/p[text()='Coming soon#{[8230].pack('U*')}']), output, 1
+      assert_xpath %(//tbody/tr/td[2]/p[text()='Coming soon#{expand_entity 8230}#{expand_entity 8203}']), output, 1
     end
 
     test 'table and col width not assigned when autowidth option is specified' do
@@ -159,7 +207,7 @@ A | here| a | there
       assert_css 'table > tbody > tr:nth-child(2) > td', output, 4
     end
 
-    test 'colspec attribute sets number of columns' do
+    test 'colspec attribute using asterisk syntax sets number of columns' do
       input = <<-EOS
 [cols="3*"]
 |===
@@ -185,7 +233,7 @@ A | here| a | there
       assert_css 'table > tbody > tr', output, 2
     end
 
-    test 'table with explicit deprecated syntax column count can have multiple rows on a single line' do
+    test 'table with explicit deprecated colspec syntax can have multiple rows on a single line' do
       input = <<-EOS
 [cols="3"]
 |===
@@ -199,15 +247,73 @@ A | here| a | there
       assert_css 'table > tbody > tr', output, 2
     end
 
+    test 'columns are added for empty records in colspec attribute' do
+      input = <<-EOS
+[cols="<,"]
+|===
+|one |two
+|1 |2 |a |b
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > tbody > tr', output, 3
+    end
+
+    test 'cols attribute may include spaces' do
+      input = <<-EOS
+[cols=" 1, 1 "]
+|===
+|one |two |1 |2 |a |b
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'col[style="width: 50%;"]', output, 2
+      assert_css 'table > tbody > tr', output, 3
+    end
+
+    test 'blank cols attribute should be ignored' do
+      input = <<-EOS
+[cols=" "]
+|===
+|one |two
+|1 |2 |a |b
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'col[style="width: 50%;"]', output, 2
+      assert_css 'table > tbody > tr', output, 3
+    end
+
+    test 'empty cols attribute should be ignored' do
+      input = <<-EOS
+[cols=""]
+|===
+|one |two
+|1 |2 |a |b
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'col[style="width: 50%;"]', output, 2
+      assert_css 'table > tbody > tr', output, 3
+    end
+
     test 'table with header and footer' do
       input = <<-EOS
 [frame="topbot",options="header,footer"]
 |===
 |Item       |Quantity
-|Item 1     |1        
-|Item 2     |2        
-|Item 3     |3        
-|Total      |6        
+|Item 1     |1
+|Item 2     |2
+|Item 3     |3
+|Total      |6
 |===
       EOS
       output = render_embedded_string input
@@ -223,6 +329,39 @@ A | here| a | there
       assert_css 'table > tbody > tr', output, 3
     end
 
+    test 'table with header and footer docbook' do
+      input = <<-EOS
+.Table with header, body and footer
+[frame="topbot",options="header,footer"]
+|===
+|Item       |Quantity
+|Item 1     |1
+|Item 2     |2
+|Item 3     |3
+|Total      |6
+|===
+      EOS
+      output = render_embedded_string input, :backend => 'docbook'
+      assert_css 'table', output, 1
+      assert_css 'table[frame="topbot"]', output, 1
+      assert_css 'table > title', output, 1
+      assert_css 'table > tgroup', output, 1
+      assert_css 'table > tgroup[cols="2"]', output, 1
+      assert_css 'table > tgroup[cols="2"] > colspec', output, 2
+      assert_css 'table > tgroup[cols="2"] > colspec[colwidth="50*"]', output, 2
+      assert_css 'table > tgroup > thead', output, 1
+      assert_css 'table > tgroup > thead > row', output, 1
+      assert_css 'table > tgroup > thead > row > entry', output, 2
+      assert_css 'table > tgroup > thead > row > entry > simpara', output, 0
+      assert_css 'table > tgroup > tfoot', output, 1
+      assert_css 'table > tgroup > tfoot > row', output, 1
+      assert_css 'table > tgroup > tfoot > row > entry', output, 2
+      assert_css 'table > tgroup > tfoot > row > entry > simpara', output, 2
+      assert_css 'table > tgroup > tbody', output, 1
+      assert_css 'table > tgroup > tbody > row', output, 3
+      assert_css 'table > tgroup > tbody > row', output, 3
+    end
+
     test 'table with implicit header row' do
       input = <<-EOS
 |===
@@ -321,17 +460,41 @@ A | here| a | there
       assert_css 'table > thead > tr > th', output, 3
       assert_css 'table > thead > tr > th > *', output, 0
 
-      assert_css 'table > tfoot > tr > td', output, 3
-      assert_css 'table > tfoot > tr > td > p.header', output, 1
+      assert_css 'table > tfoot > tr > th', output, 1
+      assert_css 'table > tfoot > tr > td', output, 2
       assert_css 'table > tfoot > tr > td > p > strong', output, 1
       assert_css 'table > tfoot > tr > td > p > em', output, 1
 
-      assert_css 'table > tbody > tr > td', output, 3
-      assert_css 'table > tbody > tr > td > p.header', output, 1
+      assert_css 'table > tbody > tr > th', output, 1
+      assert_css 'table > tbody > tr > td', output, 2
+      assert_css 'table > tbody > tr > td > p.header', output, 0
       assert_css 'table > tbody > tr > td > p > strong', output, 1
       assert_css 'table > tbody > tr > td > p > em > a', output, 1
     end
 
+    test 'vertical table headers use th element instead of header class' do
+      input = <<-EOS
+[cols="1h,1s,1e"]
+|====
+
+|Name |Occupation| Website
+
+|Octocat |Social coding| http://github.com
+
+|Name |Occupation| Website
+
+|====
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > tbody > tr > th', output, 3
+      assert_css 'table > tbody > tr > td', output, 6
+      assert_css 'table > tbody > tr .header', output, 0
+      assert_css 'table > tbody > tr > td > p > strong', output, 3
+      assert_css 'table > tbody > tr > td > p > em', output, 3
+      assert_css 'table > tbody > tr > td > p > em > a', output, 1
+    end
+
     test 'supports horizontal and vertical source data with blank lines and table header' do
       input = <<-EOS
 .Horizontal and vertical source data
@@ -356,13 +519,13 @@ I am getting in shape!
       EOS
       output = render_embedded_string input
       assert_css 'table', output, 1
-      assert_css 'table[style*="width:80%"]', output, 1
+      assert_css 'table[style*="width: 80%"]', output, 1
       assert_xpath '/table/caption[@class="title"][text()="Table 1. Horizontal and vertical source data"]', output, 1
       assert_css 'table > colgroup > col', output, 4
-      assert_css 'table > colgroup > col:nth-child(1)[@style*="width:17%"]', output, 1
-      assert_css 'table > colgroup > col:nth-child(2)[@style*="width:11%"]', output, 1
-      assert_css 'table > colgroup > col:nth-child(3)[@style*="width:11%"]', output, 1
-      assert_css 'table > colgroup > col:nth-child(4)[@style*="width:58%"]', output, 1
+      assert_css 'table > colgroup > col:nth-child(1)[@style*="width: 17.647%"]', output, 1
+      assert_css 'table > colgroup > col:nth-child(2)[@style*="width: 11.7647%"]', output, 1
+      assert_css 'table > colgroup > col:nth-child(3)[@style*="width: 11.7647%"]', output, 1
+      assert_css 'table > colgroup > col:nth-child(4)[@style*="width: 58.8236%"]', output, 1
       assert_css 'table > thead', output, 1
       assert_css 'table > thead > tr', output, 1
       assert_css 'table > thead > tr > th', output, 4
@@ -377,7 +540,7 @@ I am getting in shape!
 
     test 'percentages as column widths' do
       input = <<-EOS
-[width="100%", cols="<.^10%,<90%"]
+[cols="<.^10%,<90%"]
 |===
 |column A |column B
 |===
@@ -385,8 +548,8 @@ I am getting in shape!
 
       output = render_embedded_string input
       assert_xpath '/table/colgroup/col', output, 2
-      assert_xpath '(/table/colgroup/col)[1][@style="width:10%;"]', output, 1
-      assert_xpath '(/table/colgroup/col)[2][@style="width:90%;"]', output, 1
+      assert_xpath '(/table/colgroup/col)[1][@style="width: 10%;"]', output, 1
+      assert_xpath '(/table/colgroup/col)[2][@style="width: 90%;"]', output, 1
     end
 
     test 'spans, alignments and styles' do
@@ -401,14 +564,14 @@ d|9 2+>|10
       EOS
       output = render_embedded_string input
       assert_css 'table', output, 1
-      assert_css 'table > colgroup > col[style*="width:25%"]', output, 4
+      assert_css 'table > colgroup > col[style*="width: 25%"]', output, 4
       assert_css 'table > tbody > tr', output, 4
       assert_css 'table > tbody > tr > td', output, 10
       assert_css 'table > tbody > tr:nth-child(1) > td', output, 4
       assert_css 'table > tbody > tr:nth-child(2) > td', output, 3
       assert_css 'table > tbody > tr:nth-child(3) > td', output, 1
       assert_css 'table > tbody > tr:nth-child(4) > td', output, 2
-      
+
       assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(1).halign-left.valign-top p em', output, 1
       assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(2).halign-right.valign-top p strong', output, 1
       assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(3).halign-center.valign-top p', output, 1
@@ -426,6 +589,22 @@ d|9 2+>|10
       assert_css 'table > tbody > tr:nth-child(4) > td:nth-child(2).halign-right.valign-top[colspan="2"] p code', output, 1
     end
 
+    test 'sets up columns correctly if first row has cell that spans columns' do
+      input = <<-EOS
+|===
+2+^|AAA |CCC
+|AAA |BBB |CCC
+|AAA |BBB |CCC
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table > tbody > tr:nth-child(1) > td', output, 2
+      assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(1)[colspan="2"]', output, 1
+      assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(2):not([colspan])', output, 1
+      assert_css 'table > tbody > tr:nth-child(2) > td:not([colspan])', output, 3
+      assert_css 'table > tbody > tr:nth-child(3) > td:not([colspan])', output, 3
+    end
+
     test 'supports repeating cells' do
       input = <<-EOS
 |===
@@ -455,6 +634,59 @@ d|9 2+>|10
       assert_xpath '/table/tbody/tr[3]/td[3]/p[text()="c"]', output, 1
     end
 
+    test 'calculates colnames correctly when using implicit column count and single cell with colspan' do
+      input = <<-EOS
+|===
+2+|Two Columns
+|One Column |One Column
+|===
+      EOS
+
+      output = render_embedded_string input, :backend => 'docbook'
+      assert_xpath '//colspec', output, 2
+      assert_xpath '(//colspec)[1][@colname="col_1"]', output, 1
+      assert_xpath '(//colspec)[2][@colname="col_2"]', output, 1
+      assert_xpath '//row', output, 2
+      assert_xpath '(//row)[1]/entry', output, 1
+      assert_xpath '(//row)[1]/entry[@namest="col_1"][@nameend="col_2"]', output, 1
+    end
+
+    test 'calculates colnames correctly when using implicit column count and cells with mixed colspans' do
+      input = <<-EOS
+|===
+2+|Two Columns | One Column
+|One Column |One Column |One Column
+|===
+      EOS
+
+      output = render_embedded_string input, :backend => 'docbook'
+      assert_xpath '//colspec', output, 3
+      assert_xpath '(//colspec)[1][@colname="col_1"]', output, 1
+      assert_xpath '(//colspec)[2][@colname="col_2"]', output, 1
+      assert_xpath '(//colspec)[3][@colname="col_3"]', output, 1
+      assert_xpath '//row', output, 2
+      assert_xpath '(//row)[1]/entry', output, 2
+      assert_xpath '(//row)[1]/entry[@namest="col_1"][@nameend="col_2"]', output, 1
+      assert_xpath '(//row)[2]/entry[@namest]', output, 0
+      assert_xpath '(//row)[2]/entry[@nameend]', output, 0
+    end
+
+    test 'ignores cell with colspan that exceeds colspec' do
+      input = <<-EOS
+[cols="1,1"]
+|===
+3+|A
+|B
+a|C
+
+more C
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table *', output, 0
+    end
+
     test 'paragraph, verse and literal content' do
       input = <<-EOS
 [cols=",^v,^l",options="header"]
@@ -485,10 +717,10 @@ that I can't know enough...
 that I am always hungrily operating on the margins
 of a potentially great harvest of future knowledge and wisdom.
 
-I wouldn't have it any other way. 
+I wouldn't have it any other way.
 |===
       EOS
-      output = render_embedded_string input 
+      output = render_embedded_string input
       assert_css 'table', output, 1
       assert_css 'table > colgroup > col', output, 3
       assert_css 'table > thead', output, 1
@@ -506,6 +738,54 @@ I wouldn't have it any other way.
       assert_equal 26, literal.text.lines.entries.size
     end
 
+    test 'basic asciidoc cell' do
+      input = <<-EOS
+|===
+a|--
+NOTE: content
+
+content
+--
+|===
+      EOS
+
+      result = render_embedded_string input
+      assert_css 'table.tableblock', result, 1
+      assert_css 'table.tableblock td.tableblock', result, 1
+      assert_css 'table.tableblock td.tableblock .openblock', result, 1
+      assert_css 'table.tableblock td.tableblock .openblock .admonitionblock', result, 1
+      assert_css 'table.tableblock td.tableblock .openblock .paragraph', result, 1
+    end
+
+    test 'doctype can be set in asciidoc table cell' do
+      input = <<-EOS
+|===
+a|
+:doctype: inline
+
+content
+|===
+      EOS
+
+      result = render_embedded_string input
+      assert_css 'table.tableblock', result, 1
+      assert_css 'table.tableblock .paragraph', result, 0
+    end
+
+    test 'compat mode can be activated in asciidoc table cell' do
+      input = <<-EOS
+|===
+a|
+:compat-mode:
+
+'italic'
+|===
+      EOS
+
+      result = render_embedded_string input
+      assert_css 'table.tableblock td em', result, 1
+    end
+
     test 'asciidoc content' do
       input = <<-EOS
 [cols="1e,1,5a",frame="topbot",options="header"]
@@ -536,12 +816,12 @@ output file name is used.
       table = doc.blocks.first
       assert !table.nil?
       tbody = table.rows.body
-      assert_equal 2, tbody.size  
+      assert_equal 2, tbody.size
       body_cell_1_3 = tbody[0][2]
       assert !body_cell_1_3.inner_document.nil?
       assert body_cell_1_3.inner_document.nested?
       assert_equal doc, body_cell_1_3.inner_document.parent_document
-      assert_equal doc.renderer, body_cell_1_3.inner_document.renderer
+      assert_equal doc.converter, body_cell_1_3.inner_document.converter
       output = doc.render
 
       assert_css 'table > tbody > tr', output, 2
@@ -549,7 +829,7 @@ output file name is used.
       assert_css 'table > tbody > tr:nth-child(2) > td:nth-child(3) div.dlist', output, 1
     end
 
-    test 'preprocessor directive on first line of AsciiDoc cell should be processed' do
+    test 'preprocessor directive on first line of an AsciiDoc table cell should be processed' do
       input = <<-EOS
 |===
 a|include::fixtures/include-file.asciidoc[]
@@ -560,6 +840,83 @@ a|include::fixtures/include-file.asciidoc[]
       assert_match(/included content/, output)
     end
 
+    test 'cross reference link in an AsciiDoc table cell should resolve to reference in main document' do
+      input = <<-EOS
+== Some
+
+|===
+a|See <<_more>>
+|===
+
+== More
+
+content
+      EOS
+
+      result = render_string input
+      assert_xpath '//a[@href="#_more"]', result, 1
+      assert_xpath '//a[@href="#_more"][text()="More"]', result, 1
+    end
+
+    test 'footnotes should not be shared between an AsciiDoc table cell and the main document' do
+      input = <<-EOS
+|===
+a|AsciiDoc footnote:[A lightweight markup language.]
+|===
+      EOS
+
+      result = render_string input
+      assert_css '#_footnote_1', result, 1
+    end
+
+    test 'callout numbers should be globally unique, including AsciiDoc table cells' do
+      input = <<-EOS
+= Document Title
+
+== Section 1
+
+|====
+a|
+[source, yaml]
+----
+key: value <1>
+----
+<1> First callout
+|====
+
+== Section 2
+
+|====
+a|
+[source, yaml]
+----
+key: value <1>
+----
+<1> Second callout
+|====
+
+== Section 3
+
+[source, yaml]
+----
+key: value <1>
+----
+<1> Third callout
+      EOS
+
+      result = render_string input, :backend => 'docbook'
+      conums = xmlnodes_at_xpath '//co', result
+      assert_equal 3, conums.size
+      ['CO1-1', 'CO2-1', 'CO3-1'].each_with_index do |conum, idx|
+        assert_equal conum, conums[idx].attribute('xml:id').value
+      end
+      callouts = xmlnodes_at_xpath '//callout', result
+      assert_equal 3, callouts.size
+      ['CO1-1', 'CO2-1', 'CO3-1'].each_with_index do |callout, idx|
+        assert_equal callout, callouts[idx].attribute('arearefs').value
+      end
+    end
+
     test 'nested table' do
       input = <<-EOS
 [cols="1,2a"]
@@ -580,7 +937,74 @@ a|include::fixtures/include-file.asciidoc[]
       assert_css 'table > tbody > tr > td:nth-child(2) table > tbody > tr > td', output, 2
     end
 
-    test 'nested document in AsciiDoc cell should not see doctitle of parent' do
+    test 'toc from parent document should not be included in an AsciiDoc table cell' do
+      input = <<-EOS
+= Document Title
+:toc:
+
+== Section A
+
+|===
+a|AsciiDoc content
+|===
+      EOS
+
+      output = render_string input
+      assert_css '.toc', output, 1
+      assert_css 'table .toc', output, 0
+    end
+
+    test 'should be able to enable toc in an AsciiDoc table cell' do
+      input = <<-EOS
+= Document Title
+
+== Section A
+
+|===
+a|
+= Subdocument Title
+:toc:
+
+== Subdocument Section A
+
+content
+|===
+      EOS
+
+      output = render_string input
+      assert_css '.toc', output, 1
+      assert_css 'table .toc', output, 1
+    end
+
+    test 'should be able to enable toc in both outer document and in an AsciiDoc table cell' do
+      input = <<-EOS
+= Document Title
+:toc:
+
+== Section A
+
+|===
+a|
+= Subdocument Title
+:toc: macro
+
+[#table-cell-toc]
+toc::[]
+
+== Subdocument Section A
+
+content
+|===
+      EOS
+
+      output = render_string input
+      assert_css '.toc', output, 2
+      assert_css '#toc', output, 1
+      assert_css 'table .toc', output, 1
+      assert_css 'table #table-cell-toc', output, 1
+    end
+
+    test 'document in an AsciiDoc table cell should not see doctitle of parent' do
       input = <<-EOS
 = Document Title
 
@@ -611,10 +1035,10 @@ plain
       EOS
 
       output = render_embedded_string input
-      assert_xpath '(/table/thead/tr/th)[1][@style="background-color:green;"]', output, 1
-      assert_xpath '(/table/thead/tr/th)[2][@style="background-color:green;"]', output, 0
-      assert_xpath '(/table/tbody/tr/td)[1][@style="background-color:red;"]', output, 1
-      assert_xpath '(/table/tbody/tr/td)[2][@style="background-color:green;"]', output, 0
+      assert_xpath '(/table/thead/tr/th)[1][@style="background-color: green;"]', output, 1
+      assert_xpath '(/table/thead/tr/th)[2][@style="background-color: green;"]', output, 0
+      assert_xpath '(/table/tbody/tr/td)[1][@style="background-color: red;"]', output, 1
+      assert_xpath '(/table/tbody/tr/td)[2][@style="background-color: green;"]', output, 0
     end
   end
 
@@ -632,9 +1056,13 @@ sshd:x:74:74:Privilege-separated SSH:/var/empty/sshd:/sbin/nologin
 nobody:x:99:99:Nobody:/:/sbin/nologin
 |===
       EOS
-      output = render_embedded_string input
+      doc = document_from_string input, :header_footer => false
+      table = doc.blocks[0]
+      assert 100, table.columns.map {|col| col.attributes['colpcwidth'] }.reduce(:+)
+      output = doc.convert
       assert_css 'table', output, 1
-      assert_css 'table > colgroup > col[style*="width:14%"]', output, 7
+      assert_css 'table > colgroup > col[style*="width: 14.2857"]', output, 6
+      assert_css 'table > colgroup > col:last-of-type[style*="width: 14.2858%"]', output, 1
       assert_css 'table > tbody > tr', output, 6
       assert_xpath '//tr[4]/td[5]/p/text()', output, 0
       assert_xpath '//tr[3]/td[5]/p[text()="MySQL:Server"]', output, 1
@@ -654,10 +1082,57 @@ a:b:c
       assert_css 'table > tbody > tr:nth-child(1) > td', output, 3
       assert_css 'table > tbody > tr:nth-child(2) > td', output, 3
     end
+
+    test 'single cell in DSV table should only produce single row' do
+      input = <<-EOS
+:===
+single cell
+:===
+      EOS
+
+      output = render_embedded_string input
+      assert_css 'table td', output, 1
+    end
+
+    test 'should treat trailing colon as an empty cell' do
+      input = <<-EOS
+:====
+A1:
+B1:B2
+C1:C2
+:====
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > tbody > tr', output, 3
+      assert_xpath '/table/tbody/tr[1]/td', output, 2
+      assert_xpath '/table/tbody/tr[1]/td[1]/p[text()="A1"]', output, 1
+      assert_xpath '/table/tbody/tr[1]/td[2]/p', output, 0
+      assert_xpath '/table/tbody/tr[2]/td[1]/p[text()="B1"]', output, 1
+    end
   end
 
   context 'CSV' do
 
+    test 'should treat trailing comma as an empty cell' do
+      input = <<-EOS
+,====
+A1,
+B1,B2
+C1,C2
+,====
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > tbody > tr', output, 3
+      assert_xpath '/table/tbody/tr[1]/td', output, 2
+      assert_xpath '/table/tbody/tr[1]/td[1]/p[text()="A1"]', output, 1
+      assert_xpath '/table/tbody/tr[1]/td[2]/p', output, 0
+      assert_xpath '/table/tbody/tr[2]/td[1]/p[text()="B1"]', output, 1
+    end
+
     test 'mixed unquoted records and quoted records with escaped quotes, commas and wrapped lines' do
       input = <<-EOS
 [format="csv",options="header"]
@@ -670,9 +1145,9 @@ Year,Make,Model,Description,Price
 air, moon roof, loaded",4799.00
 |===
       EOS
-      output = render_embedded_string input 
+      output = render_embedded_string input
       assert_css 'table', output, 1
-      assert_css 'table > colgroup > col[style*="width:20%"]', output, 5
+      assert_css 'table > colgroup > col[style*="width: 20%"]', output, 5
       assert_css 'table > thead > tr', output, 1
       assert_css 'table > tbody > tr', output, 4
       assert_xpath '((//tbody/tr)[1]/td)[4]/p[text()="ac, abs, moon"]', output, 1
@@ -710,5 +1185,88 @@ a;b;c
       assert_css 'table > tbody > tr:nth-child(1) > td', output, 3
       assert_css 'table > tbody > tr:nth-child(2) > td', output, 3
     end
+
+    test 'custom separator for an AsciiDoc table cell' do
+      input = <<-EOS
+[cols=2,separator=!]
+|===
+!Pipe output to vim
+a!
+----
+asciidoctor -o - -s test.adoc | view -
+----
+|===
+      EOS
+      output = render_embedded_string input
+      assert_css 'table', output, 1
+      assert_css 'table > colgroup > col', output, 2
+      assert_css 'table > tbody > tr', output, 1
+      assert_css 'table > tbody > tr:nth-child(1) > td', output, 2
+      assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(1) p', output, 1
+      assert_css 'table > tbody > tr:nth-child(1) > td:nth-child(2) .listingblock', output, 1
+    end
+
+    test 'single cell in CSV table should only produce single row' do
+      input = <<-EOS
+,===
+single cell
+,===
+      EOS
+
+      output = render_embedded_string input
+      assert_css 'table td', output, 1
+    end
+
+    test 'table with breakable db45' do
+      input = <<-EOS
+.Table with breakable
+[options="breakable"]
+|===
+|Item       |Quantity
+|Item 1     |1
+|===
+      EOS
+      output = render_embedded_string input, :backend => 'docbook45'
+      assert output.include?('<?dbfo keep-together="auto"?>')
+    end
+
+    test 'table with breakable db5' do
+      input = <<-EOS
+.Table with breakable
+[options="breakable"]
+|===
+|Item       |Quantity
+|Item 1     |1
+|===
+      EOS
+      output = render_embedded_string input, :backend => 'docbook5'
+      assert output.include?('<?dbfo keep-together="auto"?>')
+    end
+
+    test 'table with unbreakable db5' do
+      input = <<-EOS
+.Table with unbreakable
+[options="unbreakable"]
+|===
+|Item       |Quantity
+|Item 1     |1
+|===
+      EOS
+      output = render_embedded_string input, :backend => 'docbook5'
+      assert output.include?('<?dbfo keep-together="always"?>')
+    end
+
+    test 'table with unbreakable db45' do
+      input = <<-EOS
+.Table with unbreakable
+[options="unbreakable"]
+|===
+|Item       |Quantity
+|Item 1     |1
+|===
+      EOS
+      output = render_embedded_string input, :backend => 'docbook45'
+      assert output.include?('<?dbfo keep-together="always"?>')
+    end
   end
 end
